// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI6LN4AO/utils/ai_thresh_sts.c $
// $Rev: 51458 $
// $Date: 2022-08-04 11:13:02 -0500 (Thu, 04 Aug 2022) $

// 24DSI6LN4AO: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi6ln4ao_ai_thresh_sts
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI6LN4AO_IOCTL_AI_THRESH_STS service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		get		The current setting is recorded here, if not NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dsi6ln4ao_ai_thresh_sts(int fd, int index, int verbose, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	s32			ret;
	s32			set;

	if (verbose)
		gsc_label_index("AI Threshold Status", index);

	ret		= dsi6ln4ao_ioctl(fd, DSI6LN4AO_IOCTL_AI_THRESH_STS, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs	= 1;
			ptr		= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case DSI6LN4AO_AI_THRESH_STS_CLEAR:

			ptr	= "Clear";
			break;

		case DSI6LN4AO_AI_THRESH_STS_SET:

			ptr	= "Set";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


