// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI6LN4AO/utils/data_format.c $
// $Rev: 51458 $
// $Date: 2022-08-04 11:13:02 -0500 (Thu, 04 Aug 2022) $

// 24DSI6LN4AO: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi6ln4ao_data_format
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI6LN4AO_IOCTL_DATA_FORMAT service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		The setting to apply.
*
*		get		The current setting is recorded here, if not NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	dsi6ln4ao_data_format(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("Data Format", index);

	ret		= dsi6ln4ao_ioctl(fd, DSI6LN4AO_IOCTL_DATA_FORMAT, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "INVALID: %ld", (long) set);
			break;

		case DSI6LN4AO_DATA_FORMAT_2S_COMP:

			ptr	= "Twos Compliment";
			break;

		case DSI6LN4AO_DATA_FORMAT_OFF_BIN:

			ptr	= "Offset Binary";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


